import { inspect as e } from "../jsutils/inspect.mjs";

import { memoize3 as r } from "../jsutils/memoize3.mjs";

import { invariant as t } from "../jsutils/invariant.mjs";

import { isPromise as n } from "../jsutils/isPromise.mjs";

import { isObjectLike as o } from "../jsutils/isObjectLike.mjs";

import { promiseReduce as i } from "../jsutils/promiseReduce.mjs";

import { promiseForObject as a } from "../jsutils/promiseForObject.mjs";

import { pathToArray as u, addPath as l } from "../jsutils/Path.mjs";

import { isIterableObject as s } from "../jsutils/isIterableObject.mjs";

import { GraphQLError as f } from "../error/GraphQLError.mjs";

import { locatedError as c } from "../error/locatedError.mjs";

import { OperationTypeNode as m } from "../language/ast.mjs";

import { Kind as p } from "../language/kinds.mjs";

import { assertValidSchema as v } from "../type/validate.mjs";

import { SchemaMetaFieldDef as d, TypeMetaFieldDef as h, TypeNameMetaFieldDef as y } from "../type/introspection.mjs";

import { isNonNullType as b, isListType as j, isLeafType as w, isAbstractType as x, isObjectType as T } from "../type/definition.mjs";

import { getVariableValues as V, getArgumentValues as g } from "./values.mjs";

import { collectFields as E, collectSubfields as R } from "./collectFields.mjs";

var O = r((function(e, r, t) {
  return R(e.schema, e.fragments, e.variableValues, r, t);
}));

function execute(e) {
  var r = e.rootValue;
  assertValidExecutionArguments(e.schema);
  var t = buildExecutionContext(e);
  if (!("schema" in t)) {
    return {
      errors: t
    };
  }
  try {
    var o = function executeOperation(e, r, t) {
      var o = e.schema.getRootType(r.operation);
      if (null == o) {
        throw new f("Schema is not configured to execute " + r.operation + " operation.", r);
      }
      var a = E(e.schema, e.fragments, e.variableValues, o, r.selectionSet);
      var u = void 0;
      switch (r.operation) {
       case m.QUERY:
        return executeFields(e, o, t, u, a);

       case m.MUTATION:
        return function executeFieldsSerially(e, r, t, o, a) {
          return i(a.entries(), (function(i, a) {
            var u = a[0];
            var s = a[1];
            var f = l(o, u, r.name);
            var c = executeField(e, r, t, s, f);
            if (void 0 === c) {
              return i;
            }
            if (n(c)) {
              return c.then((function(e) {
                i[u] = e;
                return i;
              }));
            }
            i[u] = c;
            return i;
          }), Object.create(null));
        }(e, o, t, u, a);

       case m.SUBSCRIPTION:
        return executeFields(e, o, t, u, a);
      }
    }(t, t.operation, r);
    if (n(o)) {
      return o.then((function(e) {
        return buildResponse(e, t.errors);
      }), (function(e) {
        t.errors.push(e);
        return buildResponse(null, t.errors);
      }));
    }
    return buildResponse(o, t.errors);
  } catch (e) {
    t.errors.push(e);
    return buildResponse(null, t.errors);
  }
}

function executeSync(e) {
  var r = execute(e);
  if (n(r)) {
    throw new Error("GraphQL execution failed to complete synchronously.");
  }
  return r;
}

function buildResponse(e, r) {
  return 0 === r.length ? {
    data: e
  } : {
    errors: r,
    data: e
  };
}

function assertValidExecutionArguments(e, r, t) {
  v(e);
}

function buildExecutionContext(e) {
  var r, t;
  var n = e.schema;
  var o = e.document;
  var i = e.rootValue;
  var a = e.contextValue;
  var u = e.variableValues;
  var l = e.operationName;
  var s = e.fieldResolver;
  var c = e.typeResolver;
  var m = e.subscribeFieldResolver;
  var v;
  var d = Object.create(null);
  for (var h = 0, y = o.definitions; h < y.length; h += 1) {
    var b = y[h];
    switch (b.kind) {
     case p.OPERATION_DEFINITION:
      if (null == l) {
        if (void 0 !== v) {
          return [ new f("Must provide operation name if query contains multiple operations.") ];
        }
        v = b;
      } else if ((null === (r = b.name) || void 0 === r ? void 0 : r.value) === l) {
        v = b;
      }
      break;

     case p.FRAGMENT_DEFINITION:
      d[b.name.value] = b;
    }
  }
  if (!v) {
    if (null != l) {
      return [ new f('Unknown operation named "' + l + '".') ];
    }
    return [ new f("Must provide an operation.") ];
  }
  var j = null !== (t = v.variableDefinitions) && void 0 !== t ? t : [];
  var w = V(n, j, null != u ? u : {}, {
    maxErrors: 50
  });
  if (w.errors) {
    return w.errors;
  }
  return {
    schema: n,
    fragments: d,
    rootValue: i,
    contextValue: a,
    operation: v,
    variableValues: w.coerced,
    fieldResolver: null != s ? s : defaultFieldResolver,
    typeResolver: null != c ? c : defaultTypeResolver,
    subscribeFieldResolver: null != m ? m : defaultFieldResolver,
    errors: []
  };
}

function executeFields(e, r, t, o, i) {
  var u = Object.create(null);
  var s = !1;
  for (var f = 0, c = i.entries(); f < c.length; f += 1) {
    var m = c[f];
    var p = m[0];
    var v = executeField(e, r, t, m[1], l(o, p, r.name));
    if (void 0 !== v) {
      u[p] = v;
      if (n(v)) {
        s = !0;
      }
    }
  }
  if (!s) {
    return u;
  }
  return a(u);
}

function executeField(e, r, t, o, i) {
  var a;
  var l = getFieldDef(e.schema, r, o[0]);
  if (!l) {
    return;
  }
  var s = l.type;
  var f = null !== (a = l.resolve) && void 0 !== a ? a : e.fieldResolver;
  var m = buildResolveInfo(e, l, o, r, i);
  try {
    var p = f(t, g(l, o[0], e.variableValues), e.contextValue, m);
    var v;
    if (n(p)) {
      v = p.then((function(r) {
        return completeValue(e, s, o, m, i, r);
      }));
    } else {
      v = completeValue(e, s, o, m, i, p);
    }
    if (n(v)) {
      return v.then(void 0, (function(r) {
        return handleFieldError(c(r, o, u(i)), s, e);
      }));
    }
    return v;
  } catch (r) {
    return handleFieldError(c(r, o, u(i)), s, e);
  }
}

function buildResolveInfo(e, r, t, n, o) {
  return {
    fieldName: r.name,
    fieldNodes: t,
    returnType: r.type,
    parentType: n,
    path: o,
    schema: e.schema,
    fragments: e.fragments,
    rootValue: e.rootValue,
    operation: e.operation,
    variableValues: e.variableValues
  };
}

function handleFieldError(e, r, t) {
  if (b(r)) {
    throw e;
  }
  t.errors.push(e);
  return null;
}

function completeValue(r, o, i, a, m, p) {
  if (p instanceof Error) {
    throw p;
  }
  if (b(o)) {
    var v = completeValue(r, o.ofType, i, a, m, p);
    if (null === v) {
      throw new Error("Cannot return null for non-nullable field " + a.parentType.name + "." + a.fieldName + ".");
    }
    return v;
  }
  if (null == p) {
    return null;
  }
  if (j(o)) {
    return function completeListValue(e, r, t, o, i, a) {
      if (!s(a)) {
        throw new f('Expected Iterable, but did not find one for field "' + o.parentType.name + "." + o.fieldName + '".');
      }
      var m = r.ofType;
      var p = !1;
      var v = Array.from(a, (function(r, a) {
        var s = l(i, a, void 0);
        try {
          var f;
          if (n(r)) {
            f = r.then((function(r) {
              return completeValue(e, m, t, o, s, r);
            }));
          } else {
            f = completeValue(e, m, t, o, s, r);
          }
          if (n(f)) {
            p = !0;
            return f.then(void 0, (function(r) {
              return handleFieldError(c(r, t, u(s)), m, e);
            }));
          }
          return f;
        } catch (r) {
          return handleFieldError(c(r, t, u(s)), m, e);
        }
      }));
      return p ? Promise.all(v) : v;
    }(r, o, i, a, m, p);
  }
  if (w(o)) {
    return function completeLeafValue(r, t) {
      var n = r.serialize(t);
      if (null == n) {
        throw new Error("Expected `" + e(r) + ".serialize(" + e(t) + ")` to return non-nullable value, returned: " + e(n));
      }
      return n;
    }(o, p);
  }
  if (x(o)) {
    return function completeAbstractValue(e, r, t, o, i, a) {
      var u;
      var l = (null !== (u = r.resolveType) && void 0 !== u ? u : e.typeResolver)(a, e.contextValue, o, r);
      if (n(l)) {
        return l.then((function(n) {
          return completeObjectValue(e, ensureValidRuntimeType(n, e, r, t, o, a), t, o, i, a);
        }));
      }
      return completeObjectValue(e, ensureValidRuntimeType(l, e, r, t, o, a), t, o, i, a);
    }(r, o, i, a, m, p);
  }
  if (T(o)) {
    return completeObjectValue(r, o, i, a, m, p);
  }
  t(!1);
}

function ensureValidRuntimeType(r, t, n, o, i, a) {
  if (null == r) {
    throw new f('Abstract type "' + n.name + '" must resolve to an Object type at runtime for field "' + i.parentType.name + "." + i.fieldName + '". Either the "' + n.name + '" type should provide a "resolveType" function or each possible type should provide an "isTypeOf" function.', o);
  }
  if (T(r)) {
    throw new f("Support for returning GraphQLObjectType from resolveType was removed in graphql-js@16.0.0 please return type name instead.");
  }
  if ("string" != typeof r) {
    throw new f('Abstract type "' + n.name + '" must resolve to an Object type at runtime for field "' + i.parentType.name + "." + i.fieldName + '" with value ' + e(a) + ', received "' + e(r) + '".');
  }
  var u = t.schema.getType(r);
  if (null == u) {
    throw new f('Abstract type "' + n.name + '" was resolved to a type "' + r + '" that does not exist inside the schema.', o);
  }
  if (!T(u)) {
    throw new f('Abstract type "' + n.name + '" was resolved to a non-object type "' + r + '".', o);
  }
  if (!t.schema.isSubType(n, u)) {
    throw new f('Runtime Object type "' + u.name + '" is not a possible type for "' + n.name + '".', o);
  }
  return u;
}

function completeObjectValue(e, r, t, o, i, a) {
  var u = O(e, r, t);
  if (r.isTypeOf) {
    var l = r.isTypeOf(a, e.contextValue, o);
    if (n(l)) {
      return l.then((function(n) {
        if (!n) {
          throw invalidReturnTypeError(r, a, t);
        }
        return executeFields(e, r, a, i, u);
      }));
    }
    if (!l) {
      throw invalidReturnTypeError(r, a, t);
    }
  }
  return executeFields(e, r, a, i, u);
}

function invalidReturnTypeError(r, t, n) {
  return new f('Expected value of type "' + r.name + '" but got: ' + e(t) + ".", n);
}

var defaultTypeResolver = function(e, r, t, i) {
  if (o(e) && "string" == typeof e.__typename) {
    return e.__typename;
  }
  var a = t.schema.getPossibleTypes(i);
  var u = [];
  for (var l = 0; l < a.length; l++) {
    var s = a[l];
    if (s.isTypeOf) {
      var f = s.isTypeOf(e, r, t);
      if (n(f)) {
        u[l] = f;
      } else if (f) {
        return s.name;
      }
    }
  }
  if (u.length) {
    return Promise.all(u).then((function(e) {
      for (var r = 0; r < e.length; r++) {
        if (e[r]) {
          return a[r].name;
        }
      }
    }));
  }
};

var defaultFieldResolver = function(e, r, t, n) {
  if (o(e) || "function" == typeof e) {
    var i = e[n.fieldName];
    if ("function" == typeof i) {
      return e[n.fieldName](r, t, n);
    }
    return i;
  }
};

function getFieldDef(e, r, t) {
  var n = t.name.value;
  if (n === d.name && e.getQueryType() === r) {
    return d;
  } else if (n === h.name && e.getQueryType() === r) {
    return h;
  } else if (n === y.name) {
    return y;
  }
  return r.getFields()[n];
}

export { assertValidExecutionArguments, buildExecutionContext, buildResolveInfo, defaultFieldResolver, defaultTypeResolver, execute, executeSync, getFieldDef };
