"use strict";

var e = require("../error/syntaxError.js");

var r = require("./ast.js");

var n = require("./tokenKind.js");

var t = require("./blockString.js");

var i = require("./characterClasses.js");

var a = function Lexer(e) {
  var t = new r.Token(n.TokenKind.SOF, 0, 0, 0, 0);
  this.source = e;
  this.lastToken = t;
  this.token = t;
  this.line = 1;
  this.lineStart = 0;
};

a.prototype.advance = function advance() {
  this.lastToken = this.token;
  return this.token = this.lookahead();
};

a.prototype.lookahead = function lookahead() {
  var e = this.token;
  if (e.kind !== n.TokenKind.EOF) {
    do {
      if (e.next) {
        e = e.next;
      } else {
        var r = readNextToken(this, e.end);
        e.next = r;
        r.prev = e;
        e = r;
      }
    } while (e.kind === n.TokenKind.COMMENT);
  }
  return e;
};

function isUnicodeScalarValue(e) {
  return e >= 0 && e <= 55295 || e >= 57344 && e <= 1114111;
}

function isSupplementaryCodePoint(e, r) {
  return isLeadingSurrogate(e.charCodeAt(r)) && isTrailingSurrogate(e.charCodeAt(r + 1));
}

function isLeadingSurrogate(e) {
  return e >= 55296 && e <= 56319;
}

function isTrailingSurrogate(e) {
  return e >= 56320 && e <= 57343;
}

function printCodePointAt(e, r) {
  var t = e.source.body.codePointAt(r);
  if (void 0 === t) {
    return n.TokenKind.EOF;
  } else if (t >= 32 && t <= 126) {
    var i = String.fromCodePoint(t);
    return '"' === i ? "'\"'" : '"' + i + '"';
  }
  return "U+" + t.toString(16).toUpperCase().padStart(4, "0");
}

function createToken(e, n, t, i, a) {
  return new r.Token(n, t, i, e.line, 1 + t - e.lineStart, a);
}

function readNextToken(r, t) {
  var a = r.source.body;
  var o = a.length;
  var c = t;
  while (c < o) {
    var d = a.charCodeAt(c);
    switch (d) {
     case 65279:
     case 9:
     case 32:
     case 44:
      ++c;
      continue;

     case 10:
      ++c;
      ++r.line;
      r.lineStart = c;
      continue;

     case 13:
      if (10 === a.charCodeAt(c + 1)) {
        c += 2;
      } else {
        ++c;
      }
      ++r.line;
      r.lineStart = c;
      continue;

     case 35:
      return readComment(r, c);

     case 33:
      return createToken(r, n.TokenKind.BANG, c, c + 1);

     case 36:
      return createToken(r, n.TokenKind.DOLLAR, c, c + 1);

     case 38:
      return createToken(r, n.TokenKind.AMP, c, c + 1);

     case 40:
      return createToken(r, n.TokenKind.PAREN_L, c, c + 1);

     case 41:
      return createToken(r, n.TokenKind.PAREN_R, c, c + 1);

     case 46:
      if (46 === a.charCodeAt(c + 1) && 46 === a.charCodeAt(c + 2)) {
        return createToken(r, n.TokenKind.SPREAD, c, c + 3);
      }
      break;

     case 58:
      return createToken(r, n.TokenKind.COLON, c, c + 1);

     case 61:
      return createToken(r, n.TokenKind.EQUALS, c, c + 1);

     case 64:
      return createToken(r, n.TokenKind.AT, c, c + 1);

     case 91:
      return createToken(r, n.TokenKind.BRACKET_L, c, c + 1);

     case 93:
      return createToken(r, n.TokenKind.BRACKET_R, c, c + 1);

     case 123:
      return createToken(r, n.TokenKind.BRACE_L, c, c + 1);

     case 124:
      return createToken(r, n.TokenKind.PIPE, c, c + 1);

     case 125:
      return createToken(r, n.TokenKind.BRACE_R, c, c + 1);

     case 34:
      if (34 === a.charCodeAt(c + 1) && 34 === a.charCodeAt(c + 2)) {
        return readBlockString(r, c);
      }
      return readString(r, c);
    }
    if (i.isDigit(d) || 45 === d) {
      return readNumber(r, c, d);
    }
    if (i.isNameStart(d)) {
      return readName(r, c);
    }
    throw e.syntaxError(r.source, c, 39 === d ? "Unexpected single quote character ('), did you mean to use a double quote (\")?" : isUnicodeScalarValue(d) || isSupplementaryCodePoint(a, c) ? "Unexpected character: " + printCodePointAt(r, c) + "." : "Invalid character: " + printCodePointAt(r, c) + ".");
  }
  return createToken(r, n.TokenKind.EOF, o, o);
}

function readComment(e, r) {
  var t = e.source.body;
  var i = t.length;
  var a = r + 1;
  while (a < i) {
    var o = t.charCodeAt(a);
    if (10 === o || 13 === o) {
      break;
    }
    if (isUnicodeScalarValue(o)) {
      ++a;
    } else if (isSupplementaryCodePoint(t, a)) {
      a += 2;
    } else {
      break;
    }
  }
  return createToken(e, n.TokenKind.COMMENT, r, a, t.slice(r + 1, a));
}

function readNumber(r, t, a) {
  var o = r.source.body;
  var c = t;
  var d = a;
  var s = !1;
  if (45 === d) {
    d = o.charCodeAt(++c);
  }
  if (48 === d) {
    d = o.charCodeAt(++c);
    if (i.isDigit(d)) {
      throw e.syntaxError(r.source, c, "Invalid number, unexpected digit after 0: " + printCodePointAt(r, c) + ".");
    }
  } else {
    c = readDigits(r, c, d);
    d = o.charCodeAt(c);
  }
  if (46 === d) {
    s = !0;
    d = o.charCodeAt(++c);
    c = readDigits(r, c, d);
    d = o.charCodeAt(c);
  }
  if (69 === d || 101 === d) {
    s = !0;
    if (43 === (d = o.charCodeAt(++c)) || 45 === d) {
      d = o.charCodeAt(++c);
    }
    c = readDigits(r, c, d);
    d = o.charCodeAt(c);
  }
  if (46 === d || i.isNameStart(d)) {
    throw e.syntaxError(r.source, c, "Invalid number, expected digit but got: " + printCodePointAt(r, c) + ".");
  }
  return createToken(r, s ? n.TokenKind.FLOAT : n.TokenKind.INT, t, c, o.slice(t, c));
}

function readDigits(r, n, t) {
  if (!i.isDigit(t)) {
    throw e.syntaxError(r.source, n, "Invalid number, expected digit but got: " + printCodePointAt(r, n) + ".");
  }
  var a = r.source.body;
  var o = n + 1;
  while (i.isDigit(a.charCodeAt(o))) {
    ++o;
  }
  return o;
}

function readString(r, t) {
  var i = r.source.body;
  var a = i.length;
  var o = t + 1;
  var c = o;
  var d = "";
  while (o < a) {
    var s = i.charCodeAt(o);
    if (34 === s) {
      d += i.slice(c, o);
      return createToken(r, n.TokenKind.STRING, t, o + 1, d);
    }
    if (92 === s) {
      d += i.slice(c, o);
      var u = 117 === i.charCodeAt(o + 1) ? 123 === i.charCodeAt(o + 2) ? readEscapedUnicodeVariableWidth(r, o) : readEscapedUnicodeFixedWidth(r, o) : readEscapedCharacter(r, o);
      d += u.value;
      c = o += u.size;
      continue;
    }
    if (10 === s || 13 === s) {
      break;
    }
    if (isUnicodeScalarValue(s)) {
      ++o;
    } else if (isSupplementaryCodePoint(i, o)) {
      o += 2;
    } else {
      throw e.syntaxError(r.source, o, "Invalid character within String: " + printCodePointAt(r, o) + ".");
    }
  }
  throw e.syntaxError(r.source, o, "Unterminated string.");
}

function readEscapedUnicodeVariableWidth(r, n) {
  var t = r.source.body;
  var i = 0;
  var a = 3;
  while (a < 12) {
    var o = t.charCodeAt(n + a++);
    if (125 === o) {
      if (a < 5 || !isUnicodeScalarValue(i)) {
        break;
      }
      return {
        value: String.fromCodePoint(i),
        size: a
      };
    }
    if ((i = i << 4 | readHexDigit(o)) < 0) {
      break;
    }
  }
  throw e.syntaxError(r.source, n, 'Invalid Unicode escape sequence: "' + t.slice(n, n + a) + '".');
}

function readEscapedUnicodeFixedWidth(r, n) {
  var t = r.source.body;
  var i = read16BitHexCode(t, n + 2);
  if (isUnicodeScalarValue(i)) {
    return {
      value: String.fromCodePoint(i),
      size: 6
    };
  }
  if (isLeadingSurrogate(i)) {
    if (92 === t.charCodeAt(n + 6) && 117 === t.charCodeAt(n + 7)) {
      var a = read16BitHexCode(t, n + 8);
      if (isTrailingSurrogate(a)) {
        return {
          value: String.fromCodePoint(i, a),
          size: 12
        };
      }
    }
  }
  throw e.syntaxError(r.source, n, 'Invalid Unicode escape sequence: "' + t.slice(n, n + 6) + '".');
}

function read16BitHexCode(e, r) {
  return readHexDigit(e.charCodeAt(r)) << 12 | readHexDigit(e.charCodeAt(r + 1)) << 8 | readHexDigit(e.charCodeAt(r + 2)) << 4 | readHexDigit(e.charCodeAt(r + 3));
}

function readHexDigit(e) {
  return e >= 48 && e <= 57 ? e - 48 : e >= 65 && e <= 70 ? e - 55 : e >= 97 && e <= 102 ? e - 87 : -1;
}

function readEscapedCharacter(r, n) {
  var t = r.source.body;
  switch (t.charCodeAt(n + 1)) {
   case 34:
    return {
      value: '"',
      size: 2
    };

   case 92:
    return {
      value: "\\",
      size: 2
    };

   case 47:
    return {
      value: "/",
      size: 2
    };

   case 98:
    return {
      value: "\b",
      size: 2
    };

   case 102:
    return {
      value: "\f",
      size: 2
    };

   case 110:
    return {
      value: "\n",
      size: 2
    };

   case 114:
    return {
      value: "\r",
      size: 2
    };

   case 116:
    return {
      value: "\t",
      size: 2
    };
  }
  throw e.syntaxError(r.source, n, 'Invalid character escape sequence: "' + t.slice(n, n + 2) + '".');
}

function readBlockString(r, i) {
  var a = r.source.body;
  var o = a.length;
  var c = r.lineStart;
  var d = i + 3;
  var s = d;
  var u = "";
  var l = [];
  while (d < o) {
    var h = a.charCodeAt(d);
    if (34 === h && 34 === a.charCodeAt(d + 1) && 34 === a.charCodeAt(d + 2)) {
      u += a.slice(s, d);
      l.push(u);
      var k = createToken(r, n.TokenKind.BLOCK_STRING, i, d + 3, t.dedentBlockStringLines(l).join("\n"));
      r.line += l.length - 1;
      r.lineStart = c;
      return k;
    }
    if (92 === h && 34 === a.charCodeAt(d + 1) && 34 === a.charCodeAt(d + 2) && 34 === a.charCodeAt(d + 3)) {
      u += a.slice(s, d);
      s = d + 1;
      d += 4;
      continue;
    }
    if (10 === h || 13 === h) {
      u += a.slice(s, d);
      l.push(u);
      if (13 === h && 10 === a.charCodeAt(d + 1)) {
        d += 2;
      } else {
        ++d;
      }
      u = "";
      s = d;
      c = d;
      continue;
    }
    if (isUnicodeScalarValue(h)) {
      ++d;
    } else if (isSupplementaryCodePoint(a, d)) {
      d += 2;
    } else {
      throw e.syntaxError(r.source, d, "Invalid character within String: " + printCodePointAt(r, d) + ".");
    }
  }
  throw e.syntaxError(r.source, d, "Unterminated string.");
}

function readName(e, r) {
  var t = e.source.body;
  var a = t.length;
  var o = r + 1;
  while (o < a) {
    var c = t.charCodeAt(o);
    if (i.isNameContinue(c)) {
      ++o;
    } else {
      break;
    }
  }
  return createToken(e, n.TokenKind.NAME, r, o, t.slice(r, o));
}

exports.Lexer = a;

exports.isPunctuatorTokenKind = function isPunctuatorTokenKind(e) {
  return e === n.TokenKind.BANG || e === n.TokenKind.DOLLAR || e === n.TokenKind.AMP || e === n.TokenKind.PAREN_L || e === n.TokenKind.PAREN_R || e === n.TokenKind.SPREAD || e === n.TokenKind.COLON || e === n.TokenKind.EQUALS || e === n.TokenKind.AT || e === n.TokenKind.BRACKET_L || e === n.TokenKind.BRACKET_R || e === n.TokenKind.BRACE_L || e === n.TokenKind.PIPE || e === n.TokenKind.BRACE_R;
};
