import { syntaxError as e } from "../error/syntaxError.mjs";

import { Token as r } from "./ast.mjs";

import { TokenKind as t } from "./tokenKind.mjs";

import { dedentBlockStringLines as a } from "./blockString.mjs";

import { isDigit as i, isNameStart as n, isNameContinue as o } from "./characterClasses.mjs";

var c = function Lexer(e) {
  var a = new r(t.SOF, 0, 0, 0, 0);
  this.source = e;
  this.lastToken = a;
  this.token = a;
  this.line = 1;
  this.lineStart = 0;
};

c.prototype.advance = function advance() {
  this.lastToken = this.token;
  return this.token = this.lookahead();
};

c.prototype.lookahead = function lookahead() {
  var e = this.token;
  if (e.kind !== t.EOF) {
    do {
      if (e.next) {
        e = e.next;
      } else {
        var r = readNextToken(this, e.end);
        e.next = r;
        r.prev = e;
        e = r;
      }
    } while (e.kind === t.COMMENT);
  }
  return e;
};

function isPunctuatorTokenKind(e) {
  return e === t.BANG || e === t.DOLLAR || e === t.AMP || e === t.PAREN_L || e === t.PAREN_R || e === t.SPREAD || e === t.COLON || e === t.EQUALS || e === t.AT || e === t.BRACKET_L || e === t.BRACKET_R || e === t.BRACE_L || e === t.PIPE || e === t.BRACE_R;
}

function isUnicodeScalarValue(e) {
  return e >= 0 && e <= 55295 || e >= 57344 && e <= 1114111;
}

function isSupplementaryCodePoint(e, r) {
  return isLeadingSurrogate(e.charCodeAt(r)) && isTrailingSurrogate(e.charCodeAt(r + 1));
}

function isLeadingSurrogate(e) {
  return e >= 55296 && e <= 56319;
}

function isTrailingSurrogate(e) {
  return e >= 56320 && e <= 57343;
}

function printCodePointAt(e, r) {
  var a = e.source.body.codePointAt(r);
  if (void 0 === a) {
    return t.EOF;
  } else if (a >= 32 && a <= 126) {
    var i = String.fromCodePoint(a);
    return '"' === i ? "'\"'" : '"' + i + '"';
  }
  return "U+" + a.toString(16).toUpperCase().padStart(4, "0");
}

function createToken(e, t, a, i, n) {
  return new r(t, a, i, e.line, 1 + a - e.lineStart, n);
}

function readNextToken(r, a) {
  var o = r.source.body;
  var c = o.length;
  var d = a;
  while (d < c) {
    var s = o.charCodeAt(d);
    switch (s) {
     case 65279:
     case 9:
     case 32:
     case 44:
      ++d;
      continue;

     case 10:
      ++d;
      ++r.line;
      r.lineStart = d;
      continue;

     case 13:
      if (10 === o.charCodeAt(d + 1)) {
        d += 2;
      } else {
        ++d;
      }
      ++r.line;
      r.lineStart = d;
      continue;

     case 35:
      return readComment(r, d);

     case 33:
      return createToken(r, t.BANG, d, d + 1);

     case 36:
      return createToken(r, t.DOLLAR, d, d + 1);

     case 38:
      return createToken(r, t.AMP, d, d + 1);

     case 40:
      return createToken(r, t.PAREN_L, d, d + 1);

     case 41:
      return createToken(r, t.PAREN_R, d, d + 1);

     case 46:
      if (46 === o.charCodeAt(d + 1) && 46 === o.charCodeAt(d + 2)) {
        return createToken(r, t.SPREAD, d, d + 3);
      }
      break;

     case 58:
      return createToken(r, t.COLON, d, d + 1);

     case 61:
      return createToken(r, t.EQUALS, d, d + 1);

     case 64:
      return createToken(r, t.AT, d, d + 1);

     case 91:
      return createToken(r, t.BRACKET_L, d, d + 1);

     case 93:
      return createToken(r, t.BRACKET_R, d, d + 1);

     case 123:
      return createToken(r, t.BRACE_L, d, d + 1);

     case 124:
      return createToken(r, t.PIPE, d, d + 1);

     case 125:
      return createToken(r, t.BRACE_R, d, d + 1);

     case 34:
      if (34 === o.charCodeAt(d + 1) && 34 === o.charCodeAt(d + 2)) {
        return readBlockString(r, d);
      }
      return readString(r, d);
    }
    if (i(s) || 45 === s) {
      return readNumber(r, d, s);
    }
    if (n(s)) {
      return readName(r, d);
    }
    throw e(r.source, d, 39 === s ? "Unexpected single quote character ('), did you mean to use a double quote (\")?" : isUnicodeScalarValue(s) || isSupplementaryCodePoint(o, d) ? "Unexpected character: " + printCodePointAt(r, d) + "." : "Invalid character: " + printCodePointAt(r, d) + ".");
  }
  return createToken(r, t.EOF, c, c);
}

function readComment(e, r) {
  var a = e.source.body;
  var i = a.length;
  var n = r + 1;
  while (n < i) {
    var o = a.charCodeAt(n);
    if (10 === o || 13 === o) {
      break;
    }
    if (isUnicodeScalarValue(o)) {
      ++n;
    } else if (isSupplementaryCodePoint(a, n)) {
      n += 2;
    } else {
      break;
    }
  }
  return createToken(e, t.COMMENT, r, n, a.slice(r + 1, n));
}

function readNumber(r, a, o) {
  var c = r.source.body;
  var d = a;
  var s = o;
  var u = !1;
  if (45 === s) {
    s = c.charCodeAt(++d);
  }
  if (48 === s) {
    s = c.charCodeAt(++d);
    if (i(s)) {
      throw e(r.source, d, "Invalid number, unexpected digit after 0: " + printCodePointAt(r, d) + ".");
    }
  } else {
    d = readDigits(r, d, s);
    s = c.charCodeAt(d);
  }
  if (46 === s) {
    u = !0;
    s = c.charCodeAt(++d);
    d = readDigits(r, d, s);
    s = c.charCodeAt(d);
  }
  if (69 === s || 101 === s) {
    u = !0;
    if (43 === (s = c.charCodeAt(++d)) || 45 === s) {
      s = c.charCodeAt(++d);
    }
    d = readDigits(r, d, s);
    s = c.charCodeAt(d);
  }
  if (46 === s || n(s)) {
    throw e(r.source, d, "Invalid number, expected digit but got: " + printCodePointAt(r, d) + ".");
  }
  return createToken(r, u ? t.FLOAT : t.INT, a, d, c.slice(a, d));
}

function readDigits(r, t, a) {
  if (!i(a)) {
    throw e(r.source, t, "Invalid number, expected digit but got: " + printCodePointAt(r, t) + ".");
  }
  var n = r.source.body;
  var o = t + 1;
  while (i(n.charCodeAt(o))) {
    ++o;
  }
  return o;
}

function readString(r, a) {
  var i = r.source.body;
  var n = i.length;
  var o = a + 1;
  var c = o;
  var d = "";
  while (o < n) {
    var s = i.charCodeAt(o);
    if (34 === s) {
      d += i.slice(c, o);
      return createToken(r, t.STRING, a, o + 1, d);
    }
    if (92 === s) {
      d += i.slice(c, o);
      var u = 117 === i.charCodeAt(o + 1) ? 123 === i.charCodeAt(o + 2) ? readEscapedUnicodeVariableWidth(r, o) : readEscapedUnicodeFixedWidth(r, o) : readEscapedCharacter(r, o);
      d += u.value;
      c = o += u.size;
      continue;
    }
    if (10 === s || 13 === s) {
      break;
    }
    if (isUnicodeScalarValue(s)) {
      ++o;
    } else if (isSupplementaryCodePoint(i, o)) {
      o += 2;
    } else {
      throw e(r.source, o, "Invalid character within String: " + printCodePointAt(r, o) + ".");
    }
  }
  throw e(r.source, o, "Unterminated string.");
}

function readEscapedUnicodeVariableWidth(r, t) {
  var a = r.source.body;
  var i = 0;
  var n = 3;
  while (n < 12) {
    var o = a.charCodeAt(t + n++);
    if (125 === o) {
      if (n < 5 || !isUnicodeScalarValue(i)) {
        break;
      }
      return {
        value: String.fromCodePoint(i),
        size: n
      };
    }
    if ((i = i << 4 | readHexDigit(o)) < 0) {
      break;
    }
  }
  throw e(r.source, t, 'Invalid Unicode escape sequence: "' + a.slice(t, t + n) + '".');
}

function readEscapedUnicodeFixedWidth(r, t) {
  var a = r.source.body;
  var i = read16BitHexCode(a, t + 2);
  if (isUnicodeScalarValue(i)) {
    return {
      value: String.fromCodePoint(i),
      size: 6
    };
  }
  if (isLeadingSurrogate(i)) {
    if (92 === a.charCodeAt(t + 6) && 117 === a.charCodeAt(t + 7)) {
      var n = read16BitHexCode(a, t + 8);
      if (isTrailingSurrogate(n)) {
        return {
          value: String.fromCodePoint(i, n),
          size: 12
        };
      }
    }
  }
  throw e(r.source, t, 'Invalid Unicode escape sequence: "' + a.slice(t, t + 6) + '".');
}

function read16BitHexCode(e, r) {
  return readHexDigit(e.charCodeAt(r)) << 12 | readHexDigit(e.charCodeAt(r + 1)) << 8 | readHexDigit(e.charCodeAt(r + 2)) << 4 | readHexDigit(e.charCodeAt(r + 3));
}

function readHexDigit(e) {
  return e >= 48 && e <= 57 ? e - 48 : e >= 65 && e <= 70 ? e - 55 : e >= 97 && e <= 102 ? e - 87 : -1;
}

function readEscapedCharacter(r, t) {
  var a = r.source.body;
  switch (a.charCodeAt(t + 1)) {
   case 34:
    return {
      value: '"',
      size: 2
    };

   case 92:
    return {
      value: "\\",
      size: 2
    };

   case 47:
    return {
      value: "/",
      size: 2
    };

   case 98:
    return {
      value: "\b",
      size: 2
    };

   case 102:
    return {
      value: "\f",
      size: 2
    };

   case 110:
    return {
      value: "\n",
      size: 2
    };

   case 114:
    return {
      value: "\r",
      size: 2
    };

   case 116:
    return {
      value: "\t",
      size: 2
    };
  }
  throw e(r.source, t, 'Invalid character escape sequence: "' + a.slice(t, t + 2) + '".');
}

function readBlockString(r, i) {
  var n = r.source.body;
  var o = n.length;
  var c = r.lineStart;
  var d = i + 3;
  var s = d;
  var u = "";
  var l = [];
  while (d < o) {
    var h = n.charCodeAt(d);
    if (34 === h && 34 === n.charCodeAt(d + 1) && 34 === n.charCodeAt(d + 2)) {
      u += n.slice(s, d);
      l.push(u);
      var C = createToken(r, t.BLOCK_STRING, i, d + 3, a(l).join("\n"));
      r.line += l.length - 1;
      r.lineStart = c;
      return C;
    }
    if (92 === h && 34 === n.charCodeAt(d + 1) && 34 === n.charCodeAt(d + 2) && 34 === n.charCodeAt(d + 3)) {
      u += n.slice(s, d);
      s = d + 1;
      d += 4;
      continue;
    }
    if (10 === h || 13 === h) {
      u += n.slice(s, d);
      l.push(u);
      if (13 === h && 10 === n.charCodeAt(d + 1)) {
        d += 2;
      } else {
        ++d;
      }
      u = "";
      s = d;
      c = d;
      continue;
    }
    if (isUnicodeScalarValue(h)) {
      ++d;
    } else if (isSupplementaryCodePoint(n, d)) {
      d += 2;
    } else {
      throw e(r.source, d, "Invalid character within String: " + printCodePointAt(r, d) + ".");
    }
  }
  throw e(r.source, d, "Unterminated string.");
}

function readName(e, r) {
  var a = e.source.body;
  var i = a.length;
  var n = r + 1;
  while (n < i) {
    var c = a.charCodeAt(n);
    if (o(c)) {
      ++n;
    } else {
      break;
    }
  }
  return createToken(e, t.NAME, r, n, a.slice(r, n));
}

export { c as Lexer, isPunctuatorTokenKind };
