"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

var e = require("../jsutils/inspect.js");

var i = require("../jsutils/toObjMap.js");

var t = require("../jsutils/instanceOf.js");

var r = require("../language/directiveLocation.js");

var s = require("./assertName.js");

var o = require("./scalars.js");

var a = require("./definition.js");

function isDirective(e) {
  return t.instanceOf(e, n);
}

var n = function GraphQLDirective(e) {
  var t, r;
  this.name = s.assertName(e.name);
  this.description = e.description;
  this.locations = e.locations;
  this.isRepeatable = null !== (t = e.isRepeatable) && void 0 !== t ? t : !1;
  this.extensions = i.toObjMap(e.extensions);
  this.astNode = e.astNode;
  var o = null !== (r = e.args) && void 0 !== r ? r : {};
  this.args = a.defineArguments(o);
};

n.prototype.toConfig = function toConfig() {
  return {
    name: this.name,
    description: this.description,
    locations: this.locations,
    args: a.argsToArgsConfig(this.args),
    isRepeatable: this.isRepeatable,
    extensions: this.extensions,
    astNode: this.astNode
  };
};

n.prototype.toString = function toString() {
  return "@" + this.name;
};

n.prototype.toJSON = function toJSON() {
  return this.toString();
};

var c = new n({
  name: "include",
  description: "Directs the executor to include this field or fragment only when the `if` argument is true.",
  locations: [ r.DirectiveLocation.FIELD, r.DirectiveLocation.FRAGMENT_SPREAD, r.DirectiveLocation.INLINE_FRAGMENT ],
  args: {
    if: {
      type: new a.GraphQLNonNull(o.GraphQLBoolean),
      description: "Included when true."
    }
  }
});

var p = new n({
  name: "skip",
  description: "Directs the executor to skip this field or fragment when the `if` argument is true.",
  locations: [ r.DirectiveLocation.FIELD, r.DirectiveLocation.FRAGMENT_SPREAD, r.DirectiveLocation.INLINE_FRAGMENT ],
  args: {
    if: {
      type: new a.GraphQLNonNull(o.GraphQLBoolean),
      description: "Skipped when true."
    }
  }
});

var u = new n({
  name: "deprecated",
  description: "Marks an element of a GraphQL schema as no longer supported.",
  locations: [ r.DirectiveLocation.FIELD_DEFINITION, r.DirectiveLocation.ARGUMENT_DEFINITION, r.DirectiveLocation.INPUT_FIELD_DEFINITION, r.DirectiveLocation.ENUM_VALUE ],
  args: {
    reason: {
      type: o.GraphQLString,
      description: "Explains why this element was deprecated, usually also including a suggestion for how to access supported similar data. Formatted using the Markdown syntax, as specified by [CommonMark](https://commonmark.org/).",
      defaultValue: "No longer supported"
    }
  }
});

var h = new n({
  name: "specifiedBy",
  description: "Exposes a URL that specifies the behavior of this scalar.",
  locations: [ r.DirectiveLocation.SCALAR ],
  args: {
    url: {
      type: new a.GraphQLNonNull(o.GraphQLString),
      description: "The URL that specifies the behavior of this scalar."
    }
  }
});

var l = [ c, p, u, h ];

exports.DEFAULT_DEPRECATION_REASON = "No longer supported";

exports.GraphQLDeprecatedDirective = u;

exports.GraphQLDirective = n;

exports.GraphQLIncludeDirective = c;

exports.GraphQLSkipDirective = p;

exports.GraphQLSpecifiedByDirective = h;

exports.assertDirective = function assertDirective(i) {
  if (!isDirective(i)) {
    throw new Error("Expected " + e.inspect(i) + " to be a GraphQL directive.");
  }
  return i;
};

exports.isDirective = isDirective;

exports.isSpecifiedDirective = function isSpecifiedDirective(e) {
  return l.some((function(i) {
    return i.name === e.name;
  }));
};

exports.specifiedDirectives = l;
