import { inspect as e } from "../jsutils/inspect.mjs";

import { toObjMap as t } from "../jsutils/toObjMap.mjs";

import { instanceOf as i } from "../jsutils/instanceOf.mjs";

import { DirectiveLocation as s } from "../language/directiveLocation.mjs";

import { assertName as r } from "./assertName.mjs";

import { GraphQLBoolean as o, GraphQLString as n } from "./scalars.mjs";

import { defineArguments as a, argsToArgsConfig as c, GraphQLNonNull as p } from "./definition.mjs";

function isDirective(e) {
  return i(e, m);
}

function assertDirective(t) {
  if (!isDirective(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL directive.");
  }
  return t;
}

var m = function GraphQLDirective(e) {
  var i, s;
  this.name = r(e.name);
  this.description = e.description;
  this.locations = e.locations;
  this.isRepeatable = null !== (i = e.isRepeatable) && void 0 !== i ? i : !1;
  this.extensions = t(e.extensions);
  this.astNode = e.astNode;
  var o = null !== (s = e.args) && void 0 !== s ? s : {};
  this.args = a(o);
};

m.prototype.toConfig = function toConfig() {
  return {
    name: this.name,
    description: this.description,
    locations: this.locations,
    args: c(this.args),
    isRepeatable: this.isRepeatable,
    extensions: this.extensions,
    astNode: this.astNode
  };
};

m.prototype.toString = function toString() {
  return "@" + this.name;
};

m.prototype.toJSON = function toJSON() {
  return this.toString();
};

var d = new m({
  name: "include",
  description: "Directs the executor to include this field or fragment only when the `if` argument is true.",
  locations: [ s.FIELD, s.FRAGMENT_SPREAD, s.INLINE_FRAGMENT ],
  args: {
    if: {
      type: new p(o),
      description: "Included when true."
    }
  }
});

var h = new m({
  name: "skip",
  description: "Directs the executor to skip this field or fragment when the `if` argument is true.",
  locations: [ s.FIELD, s.FRAGMENT_SPREAD, s.INLINE_FRAGMENT ],
  args: {
    if: {
      type: new p(o),
      description: "Skipped when true."
    }
  }
});

var u = "No longer supported";

var l = new m({
  name: "deprecated",
  description: "Marks an element of a GraphQL schema as no longer supported.",
  locations: [ s.FIELD_DEFINITION, s.ARGUMENT_DEFINITION, s.INPUT_FIELD_DEFINITION, s.ENUM_VALUE ],
  args: {
    reason: {
      type: n,
      description: "Explains why this element was deprecated, usually also including a suggestion for how to access supported similar data. Formatted using the Markdown syntax, as specified by [CommonMark](https://commonmark.org/).",
      defaultValue: "No longer supported"
    }
  }
});

var f = new m({
  name: "specifiedBy",
  description: "Exposes a URL that specifies the behavior of this scalar.",
  locations: [ s.SCALAR ],
  args: {
    url: {
      type: new p(n),
      description: "The URL that specifies the behavior of this scalar."
    }
  }
});

var g = [ d, h, l, f ];

function isSpecifiedDirective(e) {
  return g.some((function(t) {
    return t.name === e.name;
  }));
}

export { u as DEFAULT_DEPRECATION_REASON, l as GraphQLDeprecatedDirective, m as GraphQLDirective, d as GraphQLIncludeDirective, h as GraphQLSkipDirective, f as GraphQLSpecifiedByDirective, assertDirective, isDirective, isSpecifiedDirective, g as specifiedDirectives };
