"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

var e = require("../jsutils/inspect.js");

var t = require("../jsutils/toObjMap.js");

var i = require("../jsutils/instanceOf.js");

var s = require("../language/ast.js");

var n = require("./introspection.js");

var r = require("./directives.js");

var o = require("./definition.js");

var p = function GraphQLSchema(e) {
  var i, s;
  this.__validationErrors = !0 === e.assumeValid ? [] : void 0;
  this.description = e.description;
  this.extensions = t.toObjMap(e.extensions);
  this.astNode = e.astNode;
  this.extensionASTNodes = null !== (i = e.extensionASTNodes) && void 0 !== i ? i : [];
  this._queryType = e.query;
  this._mutationType = e.mutation;
  this._subscriptionType = e.subscription;
  this._directives = null !== (s = e.directives) && void 0 !== s ? s : r.specifiedDirectives;
  var p = new Set(e.types);
  if (null != e.types) {
    for (var a = 0, c = e.types; a < c.length; a += 1) {
      var u = c[a];
      p.delete(u);
      collectReferencedTypes(u, p);
    }
  }
  if (null != this._queryType) {
    collectReferencedTypes(this._queryType, p);
  }
  if (null != this._mutationType) {
    collectReferencedTypes(this._mutationType, p);
  }
  if (null != this._subscriptionType) {
    collectReferencedTypes(this._subscriptionType, p);
  }
  for (var y = 0, l = this._directives; y < l.length; y += 1) {
    var f = l[y];
    if (r.isDirective(f)) {
      for (var h = 0, T = f.args; h < T.length; h += 1) {
        collectReferencedTypes(T[h].type, p);
      }
    }
  }
  collectReferencedTypes(n.__Schema, p);
  this._typeMap = Object.create(null);
  this._subTypeMap = Object.create(null);
  this._implementationsMap = Object.create(null);
  for (var v = 0, g = p; v < g.length; v += 1) {
    var d = g[v];
    if (null == d) {
      continue;
    }
    var m = d.name;
    if (void 0 !== this._typeMap[m]) {
      throw new Error('Schema must contain uniquely named types but contains multiple types named "' + m + '".');
    }
    this._typeMap[m] = d;
    if (o.isInterfaceType(d)) {
      for (var b = 0, _ = d.getInterfaces(); b < _.length; b += 1) {
        var j = _[b];
        if (o.isInterfaceType(j)) {
          var M = this._implementationsMap[j.name];
          if (void 0 === M) {
            M = this._implementationsMap[j.name] = {
              objects: [],
              interfaces: []
            };
          }
          M.interfaces.push(d);
        }
      }
    } else if (o.isObjectType(d)) {
      for (var O = 0, S = d.getInterfaces(); O < S.length; O += 1) {
        var R = S[O];
        if (o.isInterfaceType(R)) {
          var I = this._implementationsMap[R.name];
          if (void 0 === I) {
            I = this._implementationsMap[R.name] = {
              objects: [],
              interfaces: []
            };
          }
          I.objects.push(d);
        }
      }
    }
  }
};

p.prototype.getQueryType = function getQueryType() {
  return this._queryType;
};

p.prototype.getMutationType = function getMutationType() {
  return this._mutationType;
};

p.prototype.getSubscriptionType = function getSubscriptionType() {
  return this._subscriptionType;
};

p.prototype.getRootType = function getRootType(e) {
  switch (e) {
   case s.OperationTypeNode.QUERY:
    return this.getQueryType();

   case s.OperationTypeNode.MUTATION:
    return this.getMutationType();

   case s.OperationTypeNode.SUBSCRIPTION:
    return this.getSubscriptionType();
  }
};

p.prototype.getTypeMap = function getTypeMap() {
  return this._typeMap;
};

p.prototype.getType = function getType(e) {
  return this.getTypeMap()[e];
};

p.prototype.getPossibleTypes = function getPossibleTypes(e) {
  return o.isUnionType(e) ? e.getTypes() : this.getImplementations(e).objects;
};

p.prototype.getImplementations = function getImplementations(e) {
  var t = this._implementationsMap[e.name];
  return null != t ? t : {
    objects: [],
    interfaces: []
  };
};

p.prototype.isSubType = function isSubType(e, t) {
  var i = this._subTypeMap[e.name];
  if (void 0 === i) {
    i = Object.create(null);
    if (o.isUnionType(e)) {
      for (var s = 0, n = e.getTypes(); s < n.length; s += 1) {
        i[n[s].name] = !0;
      }
    } else {
      var r = this.getImplementations(e);
      for (var p = 0, a = r.objects; p < a.length; p += 1) {
        i[a[p].name] = !0;
      }
      for (var c = 0, u = r.interfaces; c < u.length; c += 1) {
        i[u[c].name] = !0;
      }
    }
    this._subTypeMap[e.name] = i;
  }
  return void 0 !== i[t.name];
};

p.prototype.getDirectives = function getDirectives() {
  return this._directives;
};

p.prototype.getDirective = function getDirective(e) {
  return this.getDirectives().find((function(t) {
    return t.name === e;
  }));
};

p.prototype.toConfig = function toConfig() {
  return {
    description: this.description,
    query: this.getQueryType(),
    mutation: this.getMutationType(),
    subscription: this.getSubscriptionType(),
    types: Object.values(this.getTypeMap()),
    directives: this.getDirectives(),
    extensions: this.extensions,
    astNode: this.astNode,
    extensionASTNodes: this.extensionASTNodes,
    assumeValid: void 0 !== this.__validationErrors
  };
};

function collectReferencedTypes(e, t) {
  var i = o.getNamedType(e);
  if (!t.has(i)) {
    t.add(i);
    if (o.isUnionType(i)) {
      for (var s = 0, n = i.getTypes(); s < n.length; s += 1) {
        collectReferencedTypes(n[s], t);
      }
    } else if (o.isObjectType(i) || o.isInterfaceType(i)) {
      for (var r = 0, p = i.getInterfaces(); r < p.length; r += 1) {
        collectReferencedTypes(p[r], t);
      }
      for (var a = 0, c = Object.values(i.getFields()); a < c.length; a += 1) {
        var u = c[a];
        collectReferencedTypes(u.type, t);
        for (var y = 0, l = u.args; y < l.length; y += 1) {
          collectReferencedTypes(l[y].type, t);
        }
      }
    } else if (o.isInputObjectType(i)) {
      for (var f = 0, h = Object.values(i.getFields()); f < h.length; f += 1) {
        collectReferencedTypes(h[f].type, t);
      }
    }
  }
  return t;
}

exports.GraphQLSchema = p;

exports.assertSchema = function assertSchema(t) {
  if (!function isSchema(e) {
    return i.instanceOf(e, p);
  }(t)) {
    throw new Error("Expected " + e.inspect(t) + " to be a GraphQL schema.");
  }
  return t;
};
