import { inspect as e } from "../jsutils/inspect.mjs";

import { toObjMap as t } from "../jsutils/toObjMap.mjs";

import { instanceOf as i } from "../jsutils/instanceOf.mjs";

import { OperationTypeNode as s } from "../language/ast.mjs";

import { __Schema as n } from "./introspection.mjs";

import { specifiedDirectives as r, isDirective as o } from "./directives.mjs";

import { isInterfaceType as p, isObjectType as a, isUnionType as c, getNamedType as u, isInputObjectType as l } from "./definition.mjs";

function assertSchema(t) {
  if (!function isSchema(e) {
    return i(e, y);
  }(t)) {
    throw new Error("Expected " + e(t) + " to be a GraphQL schema.");
  }
  return t;
}

var y = function GraphQLSchema(e) {
  var i, s;
  this.__validationErrors = !0 === e.assumeValid ? [] : void 0;
  this.description = e.description;
  this.extensions = t(e.extensions);
  this.astNode = e.astNode;
  this.extensionASTNodes = null !== (i = e.extensionASTNodes) && void 0 !== i ? i : [];
  this._queryType = e.query;
  this._mutationType = e.mutation;
  this._subscriptionType = e.subscription;
  this._directives = null !== (s = e.directives) && void 0 !== s ? s : r;
  var c = new Set(e.types);
  if (null != e.types) {
    for (var u = 0, l = e.types; u < l.length; u += 1) {
      var y = l[u];
      c.delete(y);
      collectReferencedTypes(y, c);
    }
  }
  if (null != this._queryType) {
    collectReferencedTypes(this._queryType, c);
  }
  if (null != this._mutationType) {
    collectReferencedTypes(this._mutationType, c);
  }
  if (null != this._subscriptionType) {
    collectReferencedTypes(this._subscriptionType, c);
  }
  for (var f = 0, h = this._directives; f < h.length; f += 1) {
    var m = h[f];
    if (o(m)) {
      for (var T = 0, g = m.args; T < g.length; T += 1) {
        collectReferencedTypes(g[T].type, c);
      }
    }
  }
  collectReferencedTypes(n, c);
  this._typeMap = Object.create(null);
  this._subTypeMap = Object.create(null);
  this._implementationsMap = Object.create(null);
  for (var v = 0, d = c; v < d.length; v += 1) {
    var b = d[v];
    if (null == b) {
      continue;
    }
    var _ = b.name;
    if (void 0 !== this._typeMap[_]) {
      throw new Error('Schema must contain uniquely named types but contains multiple types named "' + _ + '".');
    }
    this._typeMap[_] = b;
    if (p(b)) {
      for (var j = 0, M = b.getInterfaces(); j < M.length; j += 1) {
        var S = M[j];
        if (p(S)) {
          var R = this._implementationsMap[S.name];
          if (void 0 === R) {
            R = this._implementationsMap[S.name] = {
              objects: [],
              interfaces: []
            };
          }
          R.interfaces.push(b);
        }
      }
    } else if (a(b)) {
      for (var O = 0, x = b.getInterfaces(); O < x.length; O += 1) {
        var I = x[O];
        if (p(I)) {
          var N = this._implementationsMap[I.name];
          if (void 0 === N) {
            N = this._implementationsMap[I.name] = {
              objects: [],
              interfaces: []
            };
          }
          N.objects.push(b);
        }
      }
    }
  }
};

y.prototype.getQueryType = function getQueryType() {
  return this._queryType;
};

y.prototype.getMutationType = function getMutationType() {
  return this._mutationType;
};

y.prototype.getSubscriptionType = function getSubscriptionType() {
  return this._subscriptionType;
};

y.prototype.getRootType = function getRootType(e) {
  switch (e) {
   case s.QUERY:
    return this.getQueryType();

   case s.MUTATION:
    return this.getMutationType();

   case s.SUBSCRIPTION:
    return this.getSubscriptionType();
  }
};

y.prototype.getTypeMap = function getTypeMap() {
  return this._typeMap;
};

y.prototype.getType = function getType(e) {
  return this.getTypeMap()[e];
};

y.prototype.getPossibleTypes = function getPossibleTypes(e) {
  return c(e) ? e.getTypes() : this.getImplementations(e).objects;
};

y.prototype.getImplementations = function getImplementations(e) {
  var t = this._implementationsMap[e.name];
  return null != t ? t : {
    objects: [],
    interfaces: []
  };
};

y.prototype.isSubType = function isSubType(e, t) {
  var i = this._subTypeMap[e.name];
  if (void 0 === i) {
    i = Object.create(null);
    if (c(e)) {
      for (var s = 0, n = e.getTypes(); s < n.length; s += 1) {
        i[n[s].name] = !0;
      }
    } else {
      var r = this.getImplementations(e);
      for (var o = 0, p = r.objects; o < p.length; o += 1) {
        i[p[o].name] = !0;
      }
      for (var a = 0, u = r.interfaces; a < u.length; a += 1) {
        i[u[a].name] = !0;
      }
    }
    this._subTypeMap[e.name] = i;
  }
  return void 0 !== i[t.name];
};

y.prototype.getDirectives = function getDirectives() {
  return this._directives;
};

y.prototype.getDirective = function getDirective(e) {
  return this.getDirectives().find((function(t) {
    return t.name === e;
  }));
};

y.prototype.toConfig = function toConfig() {
  return {
    description: this.description,
    query: this.getQueryType(),
    mutation: this.getMutationType(),
    subscription: this.getSubscriptionType(),
    types: Object.values(this.getTypeMap()),
    directives: this.getDirectives(),
    extensions: this.extensions,
    astNode: this.astNode,
    extensionASTNodes: this.extensionASTNodes,
    assumeValid: void 0 !== this.__validationErrors
  };
};

function collectReferencedTypes(e, t) {
  var i = u(e);
  if (!t.has(i)) {
    t.add(i);
    if (c(i)) {
      for (var s = 0, n = i.getTypes(); s < n.length; s += 1) {
        collectReferencedTypes(n[s], t);
      }
    } else if (a(i) || p(i)) {
      for (var r = 0, o = i.getInterfaces(); r < o.length; r += 1) {
        collectReferencedTypes(o[r], t);
      }
      for (var y = 0, f = Object.values(i.getFields()); y < f.length; y += 1) {
        var h = f[y];
        collectReferencedTypes(h.type, t);
        for (var m = 0, T = h.args; m < T.length; m += 1) {
          collectReferencedTypes(T[m].type, t);
        }
      }
    } else if (l(i)) {
      for (var g = 0, v = Object.values(i.getFields()); g < v.length; g += 1) {
        collectReferencedTypes(v[g].type, t);
      }
    }
  }
  return t;
}

export { y as GraphQLSchema, assertSchema };
