"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

var e = require("../jsutils/inspect.js");

var t = require("../error/GraphQLError.js");

var r = require("../language/ast.js");

var n = require("../utilities/typeComparators.js");

var a = require("./schema.js");

var i = require("./introspection.js");

var o = require("./directives.js");

var s = require("./definition.js");

function validateSchema(t) {
  a.assertSchema(t);
  if (t.__validationErrors) {
    return t.__validationErrors;
  }
  var n = new p(t);
  !function validateRootTypes(t) {
    var n = t.schema;
    var a = n.getQueryType();
    if (!a) {
      t.reportError("Query root type must be provided.", n.astNode);
    } else if (!s.isObjectType(a)) {
      var i;
      t.reportError("Query root type must be Object type, it cannot be " + e.inspect(a) + ".", null !== (i = getOperationTypeNode(n, r.OperationTypeNode.QUERY)) && void 0 !== i ? i : a.astNode);
    }
    var o = n.getMutationType();
    if (o && !s.isObjectType(o)) {
      var p;
      t.reportError("Mutation root type must be Object type if provided, it cannot be " + e.inspect(o) + ".", null !== (p = getOperationTypeNode(n, r.OperationTypeNode.MUTATION)) && void 0 !== p ? p : o.astNode);
    }
    var c = n.getSubscriptionType();
    if (c && !s.isObjectType(c)) {
      var l;
      t.reportError("Subscription root type must be Object type if provided, it cannot be " + e.inspect(c) + ".", null !== (l = getOperationTypeNode(n, r.OperationTypeNode.SUBSCRIPTION)) && void 0 !== l ? l : c.astNode);
    }
  }(n);
  !function validateDirectives(t) {
    for (var r = 0, n = t.schema.getDirectives(); r < n.length; r += 1) {
      var a = n[r];
      if (!o.isDirective(a)) {
        t.reportError("Expected directive but got: " + e.inspect(a) + ".", null == a ? void 0 : a.astNode);
        continue;
      }
      validateName(t, a);
      for (var i = 0, p = a.args; i < p.length; i += 1) {
        var c = p[i];
        validateName(t, c);
        if (!s.isInputType(c.type)) {
          t.reportError("The type of @" + a.name + "(" + c.name + ":) must be Input Type but got: " + e.inspect(c.type) + ".", c.astNode);
        }
        if (s.isRequiredArgument(c) && null != c.deprecationReason) {
          var l;
          t.reportError("Required argument @" + a.name + "(" + c.name + ":) cannot be deprecated.", [ getDeprecatedDirectiveNode(c.astNode), null === (l = c.astNode) || void 0 === l ? void 0 : l.type ]);
        }
      }
    }
  }(n);
  !function validateTypes(t) {
    var r = function createInputObjectCircularRefsValidator(e) {
      var t = Object.create(null);
      var r = [];
      var n = Object.create(null);
      return detectCycleRecursive;
      function detectCycleRecursive(a) {
        if (t[a.name]) {
          return;
        }
        t[a.name] = !0;
        n[a.name] = r.length;
        for (var i = 0, o = Object.values(a.getFields()); i < o.length; i += 1) {
          var p = o[i];
          if (s.isNonNullType(p.type) && s.isInputObjectType(p.type.ofType)) {
            var c = p.type.ofType;
            var l = n[c.name];
            r.push(p);
            if (void 0 === l) {
              detectCycleRecursive(c);
            } else {
              var d = r.slice(l);
              var u = d.map((function(e) {
                return e.name;
              })).join(".");
              e.reportError('Cannot reference Input Object "' + c.name + '" within itself through a series of non-null fields: "' + u + '".', d.map((function(e) {
                return e.astNode;
              })));
            }
            r.pop();
          }
        }
        n[a.name] = void 0;
      }
    }(t);
    var n = t.schema.getTypeMap();
    for (var a = 0, o = Object.values(n); a < o.length; a += 1) {
      var p = o[a];
      if (!s.isNamedType(p)) {
        t.reportError("Expected GraphQL named type but got: " + e.inspect(p) + ".", p.astNode);
        continue;
      }
      if (!i.isIntrospectionType(p)) {
        validateName(t, p);
      }
      if (s.isObjectType(p)) {
        validateFields(t, p);
        validateInterfaces(t, p);
      } else if (s.isInterfaceType(p)) {
        validateFields(t, p);
        validateInterfaces(t, p);
      } else if (s.isUnionType(p)) {
        validateUnionMembers(t, p);
      } else if (s.isEnumType(p)) {
        validateEnumValues(t, p);
      } else if (s.isInputObjectType(p)) {
        validateInputFields(t, p);
        r(p);
      }
    }
  }(n);
  var c = n.getErrors();
  t.__validationErrors = c;
  return c;
}

var p = function SchemaValidationContext(e) {
  this._errors = [];
  this.schema = e;
};

p.prototype.reportError = function reportError(e, r) {
  var n = Array.isArray(r) ? r.filter(Boolean) : r;
  this._errors.push(new t.GraphQLError(e, n));
};

p.prototype.getErrors = function getErrors() {
  return this._errors;
};

function getOperationTypeNode(e, t) {
  var r;
  return null === (r = [ e.astNode ].concat(e.extensionASTNodes).flatMap((function(e) {
    var t;
    return null !== (t = null == e ? void 0 : e.operationTypes) && void 0 !== t ? t : [];
  })).find((function(e) {
    return e.operation === t;
  }))) || void 0 === r ? void 0 : r.type;
}

function validateName(e, t) {
  if (t.name.startsWith("__")) {
    e.reportError('Name "' + t.name + '" must not begin with "__", which is reserved by GraphQL introspection.', t.astNode);
  }
}

function validateFields(t, r) {
  var n = Object.values(r.getFields());
  if (0 === n.length) {
    t.reportError("Type " + r.name + " must define one or more fields.", [ r.astNode ].concat(r.extensionASTNodes));
  }
  for (var a = 0, i = n; a < i.length; a += 1) {
    var o = i[a];
    validateName(t, o);
    if (!s.isOutputType(o.type)) {
      var p;
      t.reportError("The type of " + r.name + "." + o.name + " must be Output Type but got: " + e.inspect(o.type) + ".", null === (p = o.astNode) || void 0 === p ? void 0 : p.type);
    }
    for (var c = 0, l = o.args; c < l.length; c += 1) {
      var d = l[c];
      var u = d.name;
      validateName(t, d);
      if (!s.isInputType(d.type)) {
        var v;
        t.reportError("The type of " + r.name + "." + o.name + "(" + u + ":) must be Input Type but got: " + e.inspect(d.type) + ".", null === (v = d.astNode) || void 0 === v ? void 0 : v.type);
      }
      if (s.isRequiredArgument(d) && null != d.deprecationReason) {
        var m;
        t.reportError("Required argument " + r.name + "." + o.name + "(" + u + ":) cannot be deprecated.", [ getDeprecatedDirectiveNode(d.astNode), null === (m = d.astNode) || void 0 === m ? void 0 : m.type ]);
      }
    }
  }
}

function validateInterfaces(t, r) {
  var n = Object.create(null);
  for (var a = 0, i = r.getInterfaces(); a < i.length; a += 1) {
    var o = i[a];
    if (!s.isInterfaceType(o)) {
      t.reportError("Type " + e.inspect(r) + " must only implement Interface types, it cannot implement " + e.inspect(o) + ".", getAllImplementsInterfaceNodes(r, o));
      continue;
    }
    if (r === o) {
      t.reportError("Type " + r.name + " cannot implement itself because it would create a circular reference.", getAllImplementsInterfaceNodes(r, o));
      continue;
    }
    if (n[o.name]) {
      t.reportError("Type " + r.name + " can only implement " + o.name + " once.", getAllImplementsInterfaceNodes(r, o));
      continue;
    }
    n[o.name] = !0;
    validateTypeImplementsAncestors(t, r, o);
    validateTypeImplementsInterface(t, r, o);
  }
}

function validateTypeImplementsInterface(t, r, a) {
  var i = r.getFields();
  for (var o = 0, p = Object.values(a.getFields()); o < p.length; o += 1) {
    var c = p[o];
    var l = c.name;
    var d = i[l];
    if (!d) {
      t.reportError("Interface field " + a.name + "." + l + " expected but " + r.name + " does not provide it.", [ c.astNode, r.astNode ].concat(r.extensionASTNodes));
      continue;
    }
    if (!n.isTypeSubTypeOf(t.schema, d.type, c.type)) {
      var u, v;
      t.reportError("Interface field " + a.name + "." + l + " expects type " + e.inspect(c.type) + " but " + r.name + "." + l + " is type " + e.inspect(d.type) + ".", [ null === (u = c.astNode) || void 0 === u ? void 0 : u.type, null === (v = d.astNode) || void 0 === v ? void 0 : v.type ]);
    }
    var loop = function() {
      var i = f[m];
      var o = i.name;
      var s = d.args.find((function(e) {
        return e.name === o;
      }));
      if (!s) {
        t.reportError("Interface field argument " + a.name + "." + l + "(" + o + ":) expected but " + r.name + "." + l + " does not provide it.", [ i.astNode, d.astNode ]);
        return;
      }
      if (!n.isEqualType(i.type, s.type)) {
        var p, c;
        t.reportError("Interface field argument " + a.name + "." + l + "(" + o + ":) expects type " + e.inspect(i.type) + " but " + r.name + "." + l + "(" + o + ":) is type " + e.inspect(s.type) + ".", [ null === (p = i.astNode) || void 0 === p ? void 0 : p.type, null === (c = s.astNode) || void 0 === c ? void 0 : c.type ]);
      }
    };
    for (var m = 0, f = c.args; m < f.length; m += 1) {
      loop();
    }
    var loop$1 = function() {
      var e = g[y];
      var n = e.name;
      if (!c.args.find((function(e) {
        return e.name === n;
      })) && s.isRequiredArgument(e)) {
        t.reportError("Object field " + r.name + "." + l + " includes required argument " + n + " that is missing from the Interface field " + a.name + "." + l + ".", [ e.astNode, c.astNode ]);
      }
    };
    for (var y = 0, g = d.args; y < g.length; y += 1) {
      loop$1();
    }
  }
}

function validateTypeImplementsAncestors(e, t, r) {
  var n = t.getInterfaces();
  for (var a = 0, i = r.getInterfaces(); a < i.length; a += 1) {
    var o = i[a];
    if (!n.includes(o)) {
      e.reportError(o === t ? "Type " + t.name + " cannot implement " + r.name + " because it would create a circular reference." : "Type " + t.name + " must implement " + o.name + " because it is implemented by " + r.name + ".", getAllImplementsInterfaceNodes(r, o).concat(getAllImplementsInterfaceNodes(t, r)));
    }
  }
}

function validateUnionMembers(t, r) {
  var n = r.getTypes();
  if (0 === n.length) {
    t.reportError("Union type " + r.name + " must define one or more member types.", [ r.astNode ].concat(r.extensionASTNodes));
  }
  var a = Object.create(null);
  for (var i = 0, o = n; i < o.length; i += 1) {
    var p = o[i];
    if (a[p.name]) {
      t.reportError("Union type " + r.name + " can only include type " + p.name + " once.", getUnionMemberTypeNodes(r, p.name));
      continue;
    }
    a[p.name] = !0;
    if (!s.isObjectType(p)) {
      t.reportError("Union type " + r.name + " can only include Object types, it cannot include " + e.inspect(p) + ".", getUnionMemberTypeNodes(r, String(p)));
    }
  }
}

function validateEnumValues(e, t) {
  var r = t.getValues();
  if (0 === r.length) {
    e.reportError("Enum type " + t.name + " must define one or more values.", [ t.astNode ].concat(t.extensionASTNodes));
  }
  for (var n = 0, a = r; n < a.length; n += 1) {
    validateName(e, a[n]);
  }
}

function validateInputFields(t, r) {
  var n = Object.values(r.getFields());
  if (0 === n.length) {
    t.reportError("Input Object type " + r.name + " must define one or more fields.", [ r.astNode ].concat(r.extensionASTNodes));
  }
  for (var a = 0, i = n; a < i.length; a += 1) {
    var o = i[a];
    validateName(t, o);
    if (!s.isInputType(o.type)) {
      var p;
      t.reportError("The type of " + r.name + "." + o.name + " must be Input Type but got: " + e.inspect(o.type) + ".", null === (p = o.astNode) || void 0 === p ? void 0 : p.type);
    }
    if (s.isRequiredInputField(o) && null != o.deprecationReason) {
      var c;
      t.reportError("Required input field " + r.name + "." + o.name + " cannot be deprecated.", [ getDeprecatedDirectiveNode(o.astNode), null === (c = o.astNode) || void 0 === c ? void 0 : c.type ]);
    }
  }
}

function getAllImplementsInterfaceNodes(e, t) {
  var r = e.astNode;
  var n = e.extensionASTNodes;
  return (null != r ? [ r ].concat(n) : n).flatMap((function(e) {
    var t;
    return null !== (t = e.interfaces) && void 0 !== t ? t : [];
  })).filter((function(e) {
    return e.name.value === t.name;
  }));
}

function getUnionMemberTypeNodes(e, t) {
  var r = e.astNode;
  var n = e.extensionASTNodes;
  return (null != r ? [ r ].concat(n) : n).flatMap((function(e) {
    var t;
    return null !== (t = e.types) && void 0 !== t ? t : [];
  })).filter((function(e) {
    return e.name.value === t;
  }));
}

function getDeprecatedDirectiveNode(e) {
  var t;
  return null == e ? void 0 : null === (t = e.directives) || void 0 === t ? void 0 : t.find((function(e) {
    return e.name.value === o.GraphQLDeprecatedDirective.name;
  }));
}

exports.assertValidSchema = function assertValidSchema(e) {
  var t = validateSchema(e);
  if (0 !== t.length) {
    throw new Error(t.map((function(e) {
      return e.message;
    })).join("\n\n"));
  }
};

exports.validateSchema = validateSchema;
