"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

var e = require("../language/kinds.js");

var t = require("../language/ast.js");

var i = require("../language/visitor.js");

var a = require("../type/definition.js");

var p = require("../type/introspection.js");

var n = require("./typeFromAST.js");

var r = function TypeInfo(e, t, i) {
  this._schema = e;
  this._typeStack = [];
  this._parentTypeStack = [];
  this._inputTypeStack = [];
  this._fieldDefStack = [];
  this._defaultValueStack = [];
  this._directive = null;
  this._argument = null;
  this._enumValue = null;
  this._getFieldDef = null != i ? i : getFieldDef;
  if (t) {
    if (a.isInputType(t)) {
      this._inputTypeStack.push(t);
    }
    if (a.isCompositeType(t)) {
      this._parentTypeStack.push(t);
    }
    if (a.isOutputType(t)) {
      this._typeStack.push(t);
    }
  }
};

r.prototype.getType = function getType() {
  if (this._typeStack.length > 0) {
    return this._typeStack[this._typeStack.length - 1];
  }
};

r.prototype.getParentType = function getParentType() {
  if (this._parentTypeStack.length > 0) {
    return this._parentTypeStack[this._parentTypeStack.length - 1];
  }
};

r.prototype.getInputType = function getInputType() {
  if (this._inputTypeStack.length > 0) {
    return this._inputTypeStack[this._inputTypeStack.length - 1];
  }
};

r.prototype.getParentInputType = function getParentInputType() {
  if (this._inputTypeStack.length > 1) {
    return this._inputTypeStack[this._inputTypeStack.length - 2];
  }
};

r.prototype.getFieldDef = function getFieldDef() {
  if (this._fieldDefStack.length > 0) {
    return this._fieldDefStack[this._fieldDefStack.length - 1];
  }
};

r.prototype.getDefaultValue = function getDefaultValue() {
  if (this._defaultValueStack.length > 0) {
    return this._defaultValueStack[this._defaultValueStack.length - 1];
  }
};

r.prototype.getDirective = function getDirective() {
  return this._directive;
};

r.prototype.getArgument = function getArgument() {
  return this._argument;
};

r.prototype.getEnumValue = function getEnumValue() {
  return this._enumValue;
};

r.prototype.enter = function enter(t) {
  var i = this._schema;
  switch (t.kind) {
   case e.Kind.SELECTION_SET:
    var p = a.getNamedType(this.getType());
    this._parentTypeStack.push(a.isCompositeType(p) ? p : void 0);
    break;

   case e.Kind.FIELD:
    var r = this.getParentType();
    var u;
    var s;
    if (r) {
      if (u = this._getFieldDef(i, r, t)) {
        s = u.type;
      }
    }
    this._fieldDefStack.push(u);
    this._typeStack.push(a.isOutputType(s) ? s : void 0);
    break;

   case e.Kind.DIRECTIVE:
    this._directive = i.getDirective(t.name.value);
    break;

   case e.Kind.OPERATION_DEFINITION:
    var h = i.getRootType(t.operation);
    this._typeStack.push(a.isObjectType(h) ? h : void 0);
    break;

   case e.Kind.INLINE_FRAGMENT:
   case e.Kind.FRAGMENT_DEFINITION:
    var l = t.typeCondition;
    var c = l ? n.typeFromAST(i, l) : a.getNamedType(this.getType());
    this._typeStack.push(a.isOutputType(c) ? c : void 0);
    break;

   case e.Kind.VARIABLE_DEFINITION:
    var y = n.typeFromAST(i, t.type);
    this._inputTypeStack.push(a.isInputType(y) ? y : void 0);
    break;

   case e.Kind.ARGUMENT:
    var T;
    var o;
    var f;
    var d = null !== (T = this.getDirective()) && void 0 !== T ? T : this.getFieldDef();
    if (d) {
      if (o = d.args.find((function(e) {
        return e.name === t.name.value;
      }))) {
        f = o.type;
      }
    }
    this._argument = o;
    this._defaultValueStack.push(o ? o.defaultValue : void 0);
    this._inputTypeStack.push(a.isInputType(f) ? f : void 0);
    break;

   case e.Kind.LIST:
    var _ = a.getNullableType(this.getInputType());
    var v = a.isListType(_) ? _.ofType : _;
    this._defaultValueStack.push(void 0);
    this._inputTypeStack.push(a.isInputType(v) ? v : void 0);
    break;

   case e.Kind.OBJECT_FIELD:
    var g = a.getNamedType(this.getInputType());
    var k;
    var S;
    if (a.isInputObjectType(g)) {
      if (S = g.getFields()[t.name.value]) {
        k = S.type;
      }
    }
    this._defaultValueStack.push(S ? S.defaultValue : void 0);
    this._inputTypeStack.push(a.isInputType(k) ? k : void 0);
    break;

   case e.Kind.ENUM:
    var I = a.getNamedType(this.getInputType());
    var E;
    if (a.isEnumType(I)) {
      E = I.getValue(t.value);
    }
    this._enumValue = E;
  }
};

r.prototype.leave = function leave(t) {
  switch (t.kind) {
   case e.Kind.SELECTION_SET:
    this._parentTypeStack.pop();
    break;

   case e.Kind.FIELD:
    this._fieldDefStack.pop();
    this._typeStack.pop();
    break;

   case e.Kind.DIRECTIVE:
    this._directive = null;
    break;

   case e.Kind.OPERATION_DEFINITION:
   case e.Kind.INLINE_FRAGMENT:
   case e.Kind.FRAGMENT_DEFINITION:
    this._typeStack.pop();
    break;

   case e.Kind.VARIABLE_DEFINITION:
    this._inputTypeStack.pop();
    break;

   case e.Kind.ARGUMENT:
    this._argument = null;
    this._defaultValueStack.pop();
    this._inputTypeStack.pop();
    break;

   case e.Kind.LIST:
   case e.Kind.OBJECT_FIELD:
    this._defaultValueStack.pop();
    this._inputTypeStack.pop();
    break;

   case e.Kind.ENUM:
    this._enumValue = null;
  }
};

function getFieldDef(e, t, i) {
  var n = i.name.value;
  if (n === p.SchemaMetaFieldDef.name && e.getQueryType() === t) {
    return p.SchemaMetaFieldDef;
  }
  if (n === p.TypeMetaFieldDef.name && e.getQueryType() === t) {
    return p.TypeMetaFieldDef;
  }
  if (n === p.TypeNameMetaFieldDef.name && a.isCompositeType(t)) {
    return p.TypeNameMetaFieldDef;
  }
  if (a.isObjectType(t) || a.isInterfaceType(t)) {
    return t.getFields()[n];
  }
}

exports.TypeInfo = r;

exports.visitWithTypeInfo = function visitWithTypeInfo(e, a) {
  return {
    enter: function enter() {
      var p = [], n = arguments.length;
      while (n--) {
        p[n] = arguments[n];
      }
      var r = p[0];
      e.enter(r);
      var u = i.getEnterLeaveForKind(a, r.kind).enter;
      if (u) {
        var s = u.apply(a, p);
        if (void 0 !== s) {
          e.leave(r);
          if (t.isNode(s)) {
            e.enter(s);
          }
        }
        return s;
      }
    },
    leave: function leave() {
      var t = [], p = arguments.length;
      while (p--) {
        t[p] = arguments[p];
      }
      var n = t[0];
      var r = i.getEnterLeaveForKind(a, n.kind).leave;
      var u;
      if (r) {
        u = r.apply(a, t);
      }
      e.leave(n);
      return u;
    }
  };
};
