"use strict";

Object.defineProperty(exports, "__esModule", {
  value: !0
});

var e = require("../jsutils/keyMap.js");

var n = require("../jsutils/invariant.js");

var r = require("../jsutils/naturalCompare.js");

var a = require("../language/printer.js");

var t = require("../language/visitor.js");

var i = require("../type/scalars.js");

var p = require("../type/definition.js");

var s = require("./astFromValue.js");

exports.BreakingChangeType = void 0;

!function(e) {
  e.TYPE_REMOVED = "TYPE_REMOVED";
  e.TYPE_CHANGED_KIND = "TYPE_CHANGED_KIND";
  e.TYPE_REMOVED_FROM_UNION = "TYPE_REMOVED_FROM_UNION";
  e.VALUE_REMOVED_FROM_ENUM = "VALUE_REMOVED_FROM_ENUM";
  e.REQUIRED_INPUT_FIELD_ADDED = "REQUIRED_INPUT_FIELD_ADDED";
  e.IMPLEMENTED_INTERFACE_REMOVED = "IMPLEMENTED_INTERFACE_REMOVED";
  e.FIELD_REMOVED = "FIELD_REMOVED";
  e.FIELD_CHANGED_KIND = "FIELD_CHANGED_KIND";
  e.REQUIRED_ARG_ADDED = "REQUIRED_ARG_ADDED";
  e.ARG_REMOVED = "ARG_REMOVED";
  e.ARG_CHANGED_KIND = "ARG_CHANGED_KIND";
  e.DIRECTIVE_REMOVED = "DIRECTIVE_REMOVED";
  e.DIRECTIVE_ARG_REMOVED = "DIRECTIVE_ARG_REMOVED";
  e.REQUIRED_DIRECTIVE_ARG_ADDED = "REQUIRED_DIRECTIVE_ARG_ADDED";
  e.DIRECTIVE_REPEATABLE_REMOVED = "DIRECTIVE_REPEATABLE_REMOVED";
  e.DIRECTIVE_LOCATION_REMOVED = "DIRECTIVE_LOCATION_REMOVED";
}(exports.BreakingChangeType || (exports.BreakingChangeType = {}));

exports.DangerousChangeType = void 0;

!function(e) {
  e.VALUE_ADDED_TO_ENUM = "VALUE_ADDED_TO_ENUM";
  e.TYPE_ADDED_TO_UNION = "TYPE_ADDED_TO_UNION";
  e.OPTIONAL_INPUT_FIELD_ADDED = "OPTIONAL_INPUT_FIELD_ADDED";
  e.OPTIONAL_ARG_ADDED = "OPTIONAL_ARG_ADDED";
  e.IMPLEMENTED_INTERFACE_ADDED = "IMPLEMENTED_INTERFACE_ADDED";
  e.ARG_DEFAULT_VALUE_CHANGE = "ARG_DEFAULT_VALUE_CHANGE";
}(exports.DangerousChangeType || (exports.DangerousChangeType = {}));

function findSchemaChanges(e, n) {
  return function findTypeChanges(e, n) {
    var r = [];
    var a = diff(Object.values(e.getTypeMap()), Object.values(n.getTypeMap()));
    for (var t = 0, s = a.removed; t < s.length; t += 1) {
      var o = s[t];
      r.push({
        type: exports.BreakingChangeType.TYPE_REMOVED,
        description: i.isSpecifiedScalarType(o) ? "Standard scalar " + o.name + " was removed because it is not referenced anymore." : o.name + " was removed."
      });
    }
    for (var d = 0, E = a.persisted; d < E.length; d += 1) {
      var u = E[d];
      o = u[0];
      var D = u[1];
      if (p.isEnumType(oldType$1) && p.isEnumType(D)) {
        r.push.apply(r, findEnumTypeChanges(oldType$1, D));
      } else if (p.isUnionType(oldType$1) && p.isUnionType(D)) {
        r.push.apply(r, findUnionTypeChanges(oldType$1, D));
      } else if (p.isInputObjectType(oldType$1) && p.isInputObjectType(D)) {
        r.push.apply(r, findInputObjectTypeChanges(oldType$1, D));
      } else if (p.isObjectType(oldType$1) && p.isObjectType(D)) {
        r.push.apply(r, findFieldChanges(oldType$1, D).concat(findImplementedInterfacesChanges(oldType$1, D)));
      } else if (p.isInterfaceType(oldType$1) && p.isInterfaceType(D)) {
        r.push.apply(r, findFieldChanges(oldType$1, D).concat(findImplementedInterfacesChanges(oldType$1, D)));
      } else if (oldType$1.constructor !== D.constructor) {
        r.push({
          type: exports.BreakingChangeType.TYPE_CHANGED_KIND,
          description: oldType$1.name + " changed from " + typeKindName(oldType$1) + " to " + typeKindName(D) + "."
        });
      }
    }
    return r;
  }(e, n).concat(function findDirectiveChanges(e, n) {
    var r = [];
    var a = diff(e.getDirectives(), n.getDirectives());
    for (var t = 0, i = a.removed; t < i.length; t += 1) {
      r.push({
        type: exports.BreakingChangeType.DIRECTIVE_REMOVED,
        description: i[t].name + " was removed."
      });
    }
    for (var s = 0, o = a.persisted; s < o.length; s += 1) {
      var d = o[s];
      var E = d[1];
      var u = diff(oldDirective$1.args, E.args);
      for (var D = 0, g = u.added; D < g.length; D += 1) {
        var l = g[D];
        if (p.isRequiredArgument(l)) {
          r.push({
            type: exports.BreakingChangeType.REQUIRED_DIRECTIVE_ARG_ADDED,
            description: "A required arg " + l.name + " on directive " + oldDirective$1.name + " was added."
          });
        }
      }
      for (var f = 0, y = u.removed; f < y.length; f += 1) {
        r.push({
          type: exports.BreakingChangeType.DIRECTIVE_ARG_REMOVED,
          description: y[f].name + " was removed from " + oldDirective$1.name + "."
        });
      }
      if (oldDirective$1.isRepeatable && !E.isRepeatable) {
        r.push({
          type: exports.BreakingChangeType.DIRECTIVE_REPEATABLE_REMOVED,
          description: "Repeatable flag was removed from " + oldDirective$1.name + "."
        });
      }
      for (var T = 0, _ = oldDirective$1.locations; T < _.length; T += 1) {
        var c = _[T];
        if (!E.locations.includes(c)) {
          r.push({
            type: exports.BreakingChangeType.DIRECTIVE_LOCATION_REMOVED,
            description: c + " was removed from " + oldDirective$1.name + "."
          });
        }
      }
    }
    return r;
  }(e, n));
}

function findInputObjectTypeChanges(e, n) {
  var r = [];
  var a = diff(Object.values(e.getFields()), Object.values(n.getFields()));
  for (var t = 0, i = a.added; t < i.length; t += 1) {
    if (p.isRequiredInputField(D = i[t])) {
      r.push({
        type: exports.BreakingChangeType.REQUIRED_INPUT_FIELD_ADDED,
        description: "A required field " + D.name + " on input type " + e.name + " was added."
      });
    } else {
      r.push({
        type: exports.DangerousChangeType.OPTIONAL_INPUT_FIELD_ADDED,
        description: "An optional field " + D.name + " on input type " + e.name + " was added."
      });
    }
  }
  for (var s = 0, o = a.removed; s < o.length; s += 1) {
    r.push({
      type: exports.BreakingChangeType.FIELD_REMOVED,
      description: e.name + "." + o[s].name + " was removed."
    });
  }
  for (var d = 0, E = a.persisted; d < E.length; d += 1) {
    var u = E[d];
    var D = u[1];
    if (!isChangeSafeForInputObjectFieldOrFieldArg(oldField$1.type, newField$1.type)) {
      r.push({
        type: exports.BreakingChangeType.FIELD_CHANGED_KIND,
        description: e.name + "." + oldField$1.name + " changed type from " + String(oldField$1.type) + " to " + String(newField$1.type) + "."
      });
    }
  }
  return r;
}

function findUnionTypeChanges(e, n) {
  var r = [];
  var a = diff(e.getTypes(), n.getTypes());
  for (var t = 0, i = a.added; t < i.length; t += 1) {
    r.push({
      type: exports.DangerousChangeType.TYPE_ADDED_TO_UNION,
      description: i[t].name + " was added to union type " + e.name + "."
    });
  }
  for (var p = 0, s = a.removed; p < s.length; p += 1) {
    r.push({
      type: exports.BreakingChangeType.TYPE_REMOVED_FROM_UNION,
      description: s[p].name + " was removed from union type " + e.name + "."
    });
  }
  return r;
}

function findEnumTypeChanges(e, n) {
  var r = [];
  var a = diff(e.getValues(), n.getValues());
  for (var t = 0, i = a.added; t < i.length; t += 1) {
    r.push({
      type: exports.DangerousChangeType.VALUE_ADDED_TO_ENUM,
      description: i[t].name + " was added to enum type " + e.name + "."
    });
  }
  for (var p = 0, s = a.removed; p < s.length; p += 1) {
    r.push({
      type: exports.BreakingChangeType.VALUE_REMOVED_FROM_ENUM,
      description: s[p].name + " was removed from enum type " + e.name + "."
    });
  }
  return r;
}

function findImplementedInterfacesChanges(e, n) {
  var r = [];
  var a = diff(e.getInterfaces(), n.getInterfaces());
  for (var t = 0, i = a.added; t < i.length; t += 1) {
    r.push({
      type: exports.DangerousChangeType.IMPLEMENTED_INTERFACE_ADDED,
      description: i[t].name + " added to interfaces implemented by " + e.name + "."
    });
  }
  for (var p = 0, s = a.removed; p < s.length; p += 1) {
    r.push({
      type: exports.BreakingChangeType.IMPLEMENTED_INTERFACE_REMOVED,
      description: e.name + " no longer implements interface " + s[p].name + "."
    });
  }
  return r;
}

function findFieldChanges(e, n) {
  var r = [];
  var a = diff(Object.values(e.getFields()), Object.values(n.getFields()));
  for (var t = 0, i = a.removed; t < i.length; t += 1) {
    r.push({
      type: exports.BreakingChangeType.FIELD_REMOVED,
      description: e.name + "." + i[t].name + " was removed."
    });
  }
  for (var p = 0, s = a.persisted; p < s.length; p += 1) {
    var o = s[p];
    var d = o[1];
    r.push.apply(r, findArgChanges(e, oldField$1, d));
    if (!isChangeSafeForObjectOrInterfaceField(oldField$1.type, d.type)) {
      r.push({
        type: exports.BreakingChangeType.FIELD_CHANGED_KIND,
        description: e.name + "." + oldField$1.name + " changed type from " + String(oldField$1.type) + " to " + String(d.type) + "."
      });
    }
  }
  return r;
}

function findArgChanges(e, n, r) {
  var a = [];
  var t = diff(n.args, r.args);
  for (var i = 0, s = t.removed; i < s.length; i += 1) {
    a.push({
      type: exports.BreakingChangeType.ARG_REMOVED,
      description: e.name + "." + n.name + " arg " + s[i].name + " was removed."
    });
  }
  for (var o = 0, d = t.persisted; o < d.length; o += 1) {
    var E = d[o];
    var u = E[1];
    if (!isChangeSafeForInputObjectFieldOrFieldArg(oldArg$1.type, u.type)) {
      a.push({
        type: exports.BreakingChangeType.ARG_CHANGED_KIND,
        description: e.name + "." + n.name + " arg " + oldArg$1.name + " has changed type from " + String(oldArg$1.type) + " to " + String(u.type) + "."
      });
    } else if (void 0 !== oldArg$1.defaultValue) {
      if (void 0 === u.defaultValue) {
        a.push({
          type: exports.DangerousChangeType.ARG_DEFAULT_VALUE_CHANGE,
          description: e.name + "." + n.name + " arg " + oldArg$1.name + " defaultValue was removed."
        });
      } else {
        var D = stringifyValue(oldArg$1.defaultValue, oldArg$1.type);
        var g = stringifyValue(u.defaultValue, u.type);
        if (D !== g) {
          a.push({
            type: exports.DangerousChangeType.ARG_DEFAULT_VALUE_CHANGE,
            description: e.name + "." + n.name + " arg " + oldArg$1.name + " has changed defaultValue from " + D + " to " + g + "."
          });
        }
      }
    }
  }
  for (var l = 0, f = t.added; l < f.length; l += 1) {
    var y = f[l];
    if (p.isRequiredArgument(y)) {
      a.push({
        type: exports.BreakingChangeType.REQUIRED_ARG_ADDED,
        description: "A required arg " + y.name + " on " + e.name + "." + n.name + " was added."
      });
    } else {
      a.push({
        type: exports.DangerousChangeType.OPTIONAL_ARG_ADDED,
        description: "An optional arg " + y.name + " on " + e.name + "." + n.name + " was added."
      });
    }
  }
  return a;
}

function isChangeSafeForObjectOrInterfaceField(e, n) {
  if (p.isListType(e)) {
    return p.isListType(n) && isChangeSafeForObjectOrInterfaceField(e.ofType, n.ofType) || p.isNonNullType(n) && isChangeSafeForObjectOrInterfaceField(e, n.ofType);
  }
  if (p.isNonNullType(e)) {
    return p.isNonNullType(n) && isChangeSafeForObjectOrInterfaceField(e.ofType, n.ofType);
  }
  return p.isNamedType(n) && e.name === n.name || p.isNonNullType(n) && isChangeSafeForObjectOrInterfaceField(e, n.ofType);
}

function isChangeSafeForInputObjectFieldOrFieldArg(e, n) {
  if (p.isListType(e)) {
    return p.isListType(n) && isChangeSafeForInputObjectFieldOrFieldArg(e.ofType, n.ofType);
  }
  if (p.isNonNullType(e)) {
    return p.isNonNullType(n) && isChangeSafeForInputObjectFieldOrFieldArg(e.ofType, n.ofType) || !p.isNonNullType(n) && isChangeSafeForInputObjectFieldOrFieldArg(e.ofType, n);
  }
  return p.isNamedType(n) && e.name === n.name;
}

function typeKindName(e) {
  if (p.isScalarType(e)) {
    return "a Scalar type";
  }
  if (p.isObjectType(e)) {
    return "an Object type";
  }
  if (p.isInterfaceType(e)) {
    return "an Interface type";
  }
  if (p.isUnionType(e)) {
    return "a Union type";
  }
  if (p.isEnumType(e)) {
    return "an Enum type";
  }
  if (p.isInputObjectType(e)) {
    return "an Input type";
  }
  n.invariant(!1);
}

function stringifyValue(e, i) {
  var p = s.astFromValue(e, i);
  null != p || n.invariant(!1);
  var o = t.visit(p, {
    ObjectValue: function ObjectValue(e) {
      var n = [].concat(e.fields);
      n.sort((function(e, n) {
        return r.naturalCompare(e.name.value, n.name.value);
      }));
      return Object.assign({}, e, {
        fields: n
      });
    }
  });
  return a.print(o);
}

function diff(n, r) {
  var a = [];
  var t = [];
  var i = [];
  var p = e.keyMap(n, (function(e) {
    return e.name;
  }));
  var s = e.keyMap(r, (function(e) {
    return e.name;
  }));
  for (var o = 0, d = n; o < d.length; o += 1) {
    var E = d[o];
    var u = s[E.name];
    if (void 0 === u) {
      t.push(E);
    } else {
      i.push([ E, u ]);
    }
  }
  for (var D = 0, g = r; D < g.length; D += 1) {
    var l = g[D];
    if (void 0 === p[l.name]) {
      a.push(l);
    }
  }
  return {
    added: a,
    persisted: i,
    removed: t
  };
}

exports.findBreakingChanges = function findBreakingChanges(e, n) {
  return findSchemaChanges(e, n).filter((function(e) {
    return e.type in exports.BreakingChangeType;
  }));
};

exports.findDangerousChanges = function findDangerousChanges(e, n) {
  return findSchemaChanges(e, n).filter((function(e) {
    return e.type in exports.DangerousChangeType;
  }));
};
