import { invariant as e } from "../../jsutils/invariant.mjs";

import { GraphQLError as r } from "../../error/GraphQLError.mjs";

import { Kind as I } from "../../language/kinds.mjs";

import { OperationTypeNode as E } from "../../language/ast.mjs";

import { DirectiveLocation as N } from "../../language/directiveLocation.mjs";

import { specifiedDirectives as t } from "../../type/directives.mjs";

function KnownDirectivesRule(n) {
  var T = Object.create(null);
  var a = n.getSchema();
  for (var i = 0, c = a ? a.getDirectives() : t; i < c.length; i += 1) {
    var _ = c[i];
    T[_.name] = _.locations;
  }
  for (var o = 0, s = n.getDocument().definitions; o < s.length; o += 1) {
    var O = s[o];
    if (O.kind === I.DIRECTIVE_DEFINITION) {
      T[O.name.value] = O.locations.map((function(e) {
        return e.value;
      }));
    }
  }
  return {
    Directive: function Directive(t, a, i, c, _) {
      var o = t.name.value;
      var s = T[o];
      if (!s) {
        n.reportError(new r('Unknown directive "@' + o + '".', t));
        return;
      }
      var O = function getDirectiveLocationForASTPath(r) {
        var t = r[r.length - 1];
        "kind" in t || e(!1);
        switch (t.kind) {
         case I.OPERATION_DEFINITION:
          return function getDirectiveLocationForOperation(e) {
            switch (e) {
             case E.QUERY:
              return N.QUERY;

             case E.MUTATION:
              return N.MUTATION;

             case E.SUBSCRIPTION:
              return N.SUBSCRIPTION;
            }
          }(t.operation);

         case I.FIELD:
          return N.FIELD;

         case I.FRAGMENT_SPREAD:
          return N.FRAGMENT_SPREAD;

         case I.INLINE_FRAGMENT:
          return N.INLINE_FRAGMENT;

         case I.FRAGMENT_DEFINITION:
          return N.FRAGMENT_DEFINITION;

         case I.VARIABLE_DEFINITION:
          return N.VARIABLE_DEFINITION;

         case I.SCHEMA_DEFINITION:
         case I.SCHEMA_EXTENSION:
          return N.SCHEMA;

         case I.SCALAR_TYPE_DEFINITION:
         case I.SCALAR_TYPE_EXTENSION:
          return N.SCALAR;

         case I.OBJECT_TYPE_DEFINITION:
         case I.OBJECT_TYPE_EXTENSION:
          return N.OBJECT;

         case I.FIELD_DEFINITION:
          return N.FIELD_DEFINITION;

         case I.INTERFACE_TYPE_DEFINITION:
         case I.INTERFACE_TYPE_EXTENSION:
          return N.INTERFACE;

         case I.UNION_TYPE_DEFINITION:
         case I.UNION_TYPE_EXTENSION:
          return N.UNION;

         case I.ENUM_TYPE_DEFINITION:
         case I.ENUM_TYPE_EXTENSION:
          return N.ENUM;

         case I.ENUM_VALUE_DEFINITION:
          return N.ENUM_VALUE;

         case I.INPUT_OBJECT_TYPE_DEFINITION:
         case I.INPUT_OBJECT_TYPE_EXTENSION:
          return N.INPUT_OBJECT;

         case I.INPUT_VALUE_DEFINITION:
          var n = r[r.length - 3];
          "kind" in n || e(!1);
          return n.kind === I.INPUT_OBJECT_TYPE_DEFINITION ? N.INPUT_FIELD_DEFINITION : N.ARGUMENT_DEFINITION;

         default:
          e(!1);
        }
      }(_);
      if (O && !s.includes(O)) {
        n.reportError(new r('Directive "@' + o + '" may not be used on ' + O + ".", t));
      }
    }
  };
}

export { KnownDirectivesRule };
